<?php
/**
 * Fabrik Admin Upgrade Model
 *
 * @package     Joomla.Administrator
 * @subpackage  Fabrik
 * @copyright   Copyright (C) 2005 Fabrik. All rights reserved.
 * @license     http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 * @since       1.6
 */

defined('_JEXEC') or die;

jimport('joomla.application.component.modeladmin');

/**
 @@@trob:
 Fabrik upgrade from 3.0 to 3.2 via scheduled task (manual run)
 create a php task, Status = UNpublished
 default: running in test mode (no updates)
 $testrun has to be set to false in scheduled task "PHP setup" ($testrun = false;) to update the database
 run manually
 displays debug info, exits
 
 changes date formats
 sets element add + edit access
 changes '_formData' to 'formData' in php code 
 
 use on your own risk
 */

$testrun = isset($testrun) ? $testrun : true;
echo "testrun = ".(int)$testrun;

		//$this->fundleMenus();
		if (!shouldUpgrade())
		{
			JFactory::getApplication()->enqueueMessage('Already updated');
			echo "<br><br>already updated"; exit;

		}
		if (backUp($testrun))
		{
			upgrade($testrun);
		}
		JFactory::getApplication()->enqueueMessage('Upgraded OK!');
		if ($testrun) {
			echo "<br><br>Testrun ok; no updates executed"; exit;
			
		}
		else {
			echo "<br><br>upgrade ok"; exit;
		}



	/**
	 * Back up the fabrik db tables and make copies of the data tables they use
	 *
	 * @return  bool
	 */

	function backUp($testrun = true)
	{
		if ($testrun) {
			return true;
		}
		
		$db = JFactory::getDbo(true);

		$tables = getFabrikTables();
		$listModel = JModelLegacy::getInstance('List', 'FabrikFEModel');
		$connModel = JModelLegacy::getInstance('Connection', 'FabrikFEModel');
		$cnnTables = array();
		foreach ($tables as $dbName => $item)
		{
			$connModel->setId($item->connection_id);
			$connModel->getConnection($item->connection_id);
			$cDb = $connModel->getDb();
			if (!array_key_exists($item->connection_id, $cnnTables))
			{
				$cnnTables[$item->connection_id] = $cDb->getTableList();
			}
			$listModel->set('connection', $connModel);
			$qTable = $cDb->quoteName('bkup30_' . $item->db_table_name);
			$qItemTable = $cDb->quoteName($item->db_table_name);

			// Drop the bkup table
			//$cDb->setQuery("DROP TABLE IF EXISTS " . $qTable);
			//if (!$cDb->query())
			//{
			//	JError::raiseError(500, $cDb->getErrorMsg());
			//	return false;
			//}
			// Test table exists
			if (!in_array($item->db_table_name, $cnnTables[$item->connection_id]))
			{
				JError::raiseNotice(500, 'backup: table not found: ' . $item->db_table_name);
				continue;
			}
			// Create the bkup table (this method will also correctly copy table indexes
			//no upgrade if bkup table already exists
			$cDb->setQuery("CREATE TABLE" . $qTable . " LIKE " . $qItemTable);
			if (!$cDb->query())
			{
				if ($cDb->getErrorNum() == 1050) {echo 'backup-tables found'; return true;}
				JError::raiseError(500, $cDb->getErrorMsg());
				return false;
			}
			$cDb->setQuery("INSERT INTO " . $qTable . " SELECT * FROM " . $qItemTable);
			if (!$cDb->query())
			{
				JError::raiseError(500, $cDb->getErrorMsg());
				return false;
			}
		}
		return true;
	}

	/**
	 * Upgrade the database to fabrik3's structure.
	 *
	 * @return  void
	 */

	function upgrade($testrun = true)
	{
		// date formats: strftime to date
		$formats = array("%a"=>"D","%A"=>"l","%d"=>"d","%e"=>"j","%u"=>"N","%w"=>"w","%W"=>"W","%b"=>"M","%h"=>"M","%B"=>"F","%m"=>"m","%G"=>"o","%y"=>"y","%Y"=>"Y","%H"=>"H","%I"=>"h","%l"=>"g","%P"=>"a","%p"=>"A","%M"=>"i","%S"=>"s");

		$db = JFactory::getDbo(true);
		$updates = array('#__fabrik_elements', '#__fabrik_cron', '#__fabrik_forms','#__fabrik_lists');
		//  '#__fabrik_groups','#__fabrik_joins', '#__fabrik_jsactions', '#__fabrik_visualizations');
		foreach ($updates as $update)
		{
			$db->setQuery("SELECT * FROM $update");
			$rows = $db->loadObjectList();
			if ($db->getErrorNum())
			{
				JError::raiseError(500, $db->getErrorMsg());
			}
			foreach ($rows as $row)
			{	
				$update_row = false;
				//params may contain code
				$param_new = str_replace('_formData','formData',$row->params,$count);
				if ($count >0) {
					echo "<br><br>formData in $update ".$row->id." ".$row->label.":<br>Param-old: ".$row->params."<br>Param-new: ".$param_new;
					$row->params = $param_new;
					$update_row = true;
				}
			
				$json = json_decode($row->params); 

				// Replace
				$p = $json;
				switch ($update)
					{
					case '#__fabrik_elements':
					
					// Elements now have add + edit acces; split old form view access
					if (!isset($p->edit_access)) {
						$p->edit_access = $row->access;
						$update_row = true;
					}
					
					// Date elements with new date formats
					if ($row->plugin == 'date') {

						$s = "<br><br>date element ".$row->id." ".$row->name. "<br>date-formats-old: list: ".$p->date_table_format." form: ".$p->date_form_format." time: ".$p->date_time_format;
						
						$p->date_table_format = str_replace(array_keys($formats), array_values($formats), $p->date_table_format,$count1); 
						$p->date_form_format = str_replace(array_keys($formats), array_values($formats), $p->date_form_format,$count2);
						$p->date_time_format = str_replace(array_keys($formats), array_values($formats), $p->date_time_format,$count3);
						
						$x =$count1+$count2+$count3;
						if ( $x > 0) {
						echo $s."<br>date-formats-new: list: ".$p->date_table_format." form: ".$p->date_form_format." time: ".$p->date_time_format;
						$update_row = true;
						}
					}
	
					break;

					}
				$row->params = json_encode($p);


				if ($update_row) {
					echo "<br>".$update.' id= '.$row->id;
					
					if (!$testrun) {
						$res = $db->updateObject($update, $row, 'id');
						echo ' updated, Status: '.$res;
					}
					else {
						echo ' Testrun';
					}
					
				}
			}
		}

	}



	/**
	 * Get all the db tables which have _fabrik_ as part of their names
	 *
	 * @return array of objects each with db_table_name and connection_id property
	 */

	function getFabrikTables()
	{
		$db = JFactory::getDbo(true);
		$r = array();
		$db->setQuery("SHOW TABLES");
		$rows = $db->loadColumn();
		foreach ($rows as $row)
		{
			if (strstr($row, '_fabrik_') && !strstr($row, 'bkup_'))
			{
				$o = new stdClass;
				$o->db_table_name = $row;
				$o->connection_id = 1;
				$r[] = $o;
			}
		}
		return $r;
	}
	/**
	 * Check for an existence of _fabrik_tables table if there is then we should upgrade
	 *
	 * @return  bool
	 */

	function shouldUpgrade()
	{
		//Fabrik3.0 elements don't have edit_access set
		$db = JFactory::getDbo(true);
		$db->setQuery("SELECT params FROM #__fabrik_elements");
		$params_example = $db->loadResult();
		$p= json_decode($params_example); 
		
		return !isset($p->edit_access);
	}

